@@ -110,4 +110,18 @@ mod tests {
             water_conditions(val).expect(val);
         }
     }
+
+    #[test]
+    fn parse_color() {
+        for val in ["WHT", "BLACKWHT", "WHT BLU"] {
+            color(val).expect(val);
+        }
+    }
+
+    #[test]
+    fn parse_whitespace() {
+        for val in [" ///// "] {
+            whitespace(val).expect(val);
+        }
+    }
 }
